#!/bin/bash 

#############################################################
# CrashPlan Linux Client Installer Script
# Silent version modified from stock script provided by Code42
#############################################################

# variables defined in install.defaults
# APP_BASENAME = human-readable application name
# DIR_BASENAME = dir name
# JRE_X64_DOWNLOAD_URL = url to the 64-bit jre
# JRE_I586_DOWNLOAD_URL = url to the 32-bit jre

log() {
	logger -it "crashplan-install" ${1}
}

SCRIPT_DIR=`dirname ${0}`
if [ ! -f "${SCRIPT_DIR}/install.defaults" ] ; then
	log "${SCRIPT_DIR}/install.defaults MISSING!"
	exit 1
fi

source "${SCRIPT_DIR}/install.defaults"

REQDBINS="grep sed cpio gzip cut head tail who"
OKJAVA="1.8"

PARENT_DIR=/usr/local
TARGETDIR=${PARENT_DIR}/${DIR_BASENAME}
BINSDIR=${PARENT_DIR}/bin
MANIFESTDIR=${PARENT_DIR}/var/${DIR_BASENAME}
INITDIR=/etc/init.d
RUNLEVEL=`who -r | sed -e 's/^.*\(run-level [0-9]\).*$/\1/' | cut -d \  -f 2`
RUNLVLDIR=/etc/rc${RUNLEVEL}.d

SRC_USER=${SUDO_USER}
if [ -z "${SRC_USER}" ] ; then
	SRC_USER=${USER}
fi

USERNAME="`id -un`"

prepdir() {
	if [ ! -d "${1}" ] ; then
		mkdir -p "${1}"
		if [ $? -ne 0 ] ; then
			log "Failed to create ${1}.  Please check your configuration."
			return 1
		fi
	fi
	return 0
}

# ===============================================================================
# Validate the environment by verifying that all necessary binaries are present
# ===============================================================================
for BIN in $REQDBINS ; do

	BIN_PATH=`which $BIN 2> /dev/null`
	if [[ $? != 0 ]]; then
		log "ERROR: $BIN not found and is required for install. Exiting"
		exit 1
	fi
done

# ===============================================================================
# Make sure to always install our JVM
# ===============================================================================
JAVACOMMON="DOWNLOAD"

# Setup ARCHIVE var to point to the cpio archive.  This will be used here to extract what we need
# to execute the Java comparison below and will be used later by the script to 
# actually extract everything.
ARCHIVE=`ls ./*_*.cpi`

# Prep target dirs, stripped down from original crashplan install script
TARGETDIR=${PARENT_DIR}/${DIR_BASENAME}
MANIFESTDIR=${TARGETDIR}/manifest
prepdir "${TARGETDIR}"
prepdir ${BINSDIR}
prepdir ${MANIFESTDIR}
prepdir ${INITDIR}
prepdir ${RUNLVLDIR}

# INSTALL TIME ===============================================

# is crashplan already there?
if [ -f ${TARGETDIR}/install.vars ]; then
	log "CrashPlan appears to already be installed in the specified location:"
	log "  ${TARGETDIR}"
	log "Please uninstall and then try this install again."
	exit 1
fi

# create a file that has our install vars so we can later uninstall
echo "" > ${TARGETDIR}/install.vars
echo "TARGETDIR=${TARGETDIR}" >> ${TARGETDIR}/install.vars
echo "BINSDIR=${BINSDIR}" >> ${TARGETDIR}/install.vars
echo "MANIFESTDIR=${MANIFESTDIR}" >> ${TARGETDIR}/install.vars
echo "INITDIR=${INITDIR}" >> ${TARGETDIR}/install.vars
echo "RUNLVLDIR=${RUNLVLDIR}" >> ${TARGETDIR}/install.vars
NOW=`date +%Y%m%d`
echo "INSTALLDATE=$NOW" >> ${TARGETDIR}/install.vars
cat ${SCRIPT_DIR}/install.defaults >> ${TARGETDIR}/install.vars

# keep track of the processor architecture
PARCH=`uname -m`

#download java
if [[ $JAVACOMMON == "DOWNLOAD" ]]; then
	if [[ $PARCH == "x86_64" ]]; then
		JVMURL="${JRE_X64_DOWNLOAD_URL}"
	else
		JVMURL="${JRE_I586_DOWNLOAD_URL}"
	fi
	JVMFILE=`basename ${JVMURL}`
	if [[ -f ${JVMFILE} ]]; then
		log "Download of the JVM found. We'll try to use it, but if it's only a partial"
		log "copy of the file then this will fail. If that happens please remove the file"
		log "and try again."
		log "JRE Archive: ${JVMFILE}"
	else
	
	    # Start by looking for wget
	    WGET_PATH=`which wget 2> /dev/null`
	    if [[ $? == 0 ]]; then
			log "    downloading the JRE using wget"
			$WGET_PATH $JVMURL
			if [[ $? != 0 ]]; then
				log "Unable to download JRE; please check network connection"
				exit 1
			fi
	    else

			CURL_PATH=`which curl 2> /dev/null`
			if [[ $? == 0 ]]; then
		    	log "    downloading the JRE using curl"
		    	$CURL_PATH -L $JVMURL -o `basename $JVMURL`
				if [[ $? != 0 ]]; then
					log "Unable to download JRE; please check network connection"
					exit 1
				fi
			else
		    	log "Could not find wget or curl.  You must install one of these utilities"
		    	log "in order to download a JVM"
		    	exit 1
			fi
	    fi
	fi

	HERE=`pwd`
	cd ${TARGETDIR}
	# Extract into ./jre
	tar -xozf "${HERE}/${JVMFILE}"
	cd "${HERE}"
	JAVACOMMON="${TARGETDIR}/jre/bin/java"
	log "Java Installed."
fi
echo "" >> ${TARGETDIR}/install.vars
echo "JAVACOMMON=${JAVACOMMON}" >> ${TARGETDIR}/install.vars

# Definition of ARCHIVE occurred above when we extracted the JAR we need to evaluate Java environment
log Unpacking ${HERE}/${ARCHIVE} ... 
HERE=`pwd`
cd ${TARGETDIR}
cat "${HERE}/${ARCHIVE}" | gzip -d -c - | cpio -i --no-preserve-owner
cd "${HERE}"

# ATOM SHELL
if [[ "$VERSION_5_UI" == "true" ]]; then
	if [[ $PARCH == "x86_64" ]]; then
		mv -v ${TARGETDIR}/electron-x64 ${TARGETDIR}/electron
		rm -rf ${TARGETDIR}/electron-ia32
	else
		mv -v ${TARGETDIR}/electron-ia32 ${TARGETDIR}/electron
		rm -rf ${TARGETDIR}/electron-x64
	fi
	mv -v ${TARGETDIR}/*.asar ${TARGETDIR}/electron/resources
	chmod 755 ${TARGETDIR}/electron/crashplan
fi

# custom?
if [ -d .Custom ]; then
  log Copying .Custom to ${TARGETDIR}
  cp -Rp .Custom "${TARGETDIR}"
fi
if [ -d custom ]; then
  log Copying custom to ${TARGETDIR}
  cp -Rp custom "${TARGETDIR}"
fi
if [ -d Custom ]; then
  log Copying custom to ${TARGETDIR}
  cp -Rp custom "${TARGETDIR}"
fi

#update the configs for file storage
if grep "<manifestPath>.*</manifestPath>" ${TARGETDIR}/conf/default.service.xml > /dev/null
	then
		sed -i "s|<manifestPath>.*</manifestPath>|<manifestPath>${MANIFESTDIR}</manifestPath>|g" ${TARGETDIR}/conf/default.service.xml
	else
		sed -i "s|<backupConfig>|<backupConfig>\n\t\t\t<manifestPath>${MANIFESTDIR}</manifestPath>|g" ${TARGETDIR}/conf/default.service.xml
fi

# the log dir
LOGDIR=${TARGETDIR}/log
chmod 777 $LOGDIR

# desktop init script
GUISCRIPT=${TARGETDIR}/bin/${APP_BASENAME}Desktop
cp scripts/${APP_BASENAME}Desktop ${GUISCRIPT}
chmod 755 ${GUISCRIPT}
#sed -imod "s|TARGETDIR=.*|TARGETDIR=${TARGETDIR}|" ${GUISCRIPT} && rm -rf ${GUISCRIPT}mod

# link to bin if appropriate
if [ -n "${BINSDIR}" ] ; then
	ln -s ${GUISCRIPT} ${BINSDIR}/${APP_BASENAME}Desktop
fi


# Install the control script for the service
INITSCRIPT=${TARGETDIR}/bin/${APP_BASENAME}Engine
cp scripts/${APP_BASENAME}Engine ${INITSCRIPT}
cp scripts/run.conf ${TARGETDIR}/bin
chmod 755 ${INITSCRIPT}

# Install the init script and modify it by applying variables currently defined in this context
# If the user is not installing as root then we install into the bin directory only.
INIT_INSTALL_DIR=${TARGETDIR}/bin
if [ -n "${INITDIR}" ] ; then
   INIT_INSTALL_DIR=${INITDIR}
fi
   
# Perform substitution on the init script; we need to make the value of INITSCRIPT available
# to what lives in /etc/init.d
SEDEXPRSUB=`echo $INITSCRIPT | sed 's/\//\\\\\//g'`
SEDEXPR="s/<INITSCRIPT>/$SEDEXPRSUB/g"
sed $SEDEXPR scripts/${DIR_BASENAME} > ${INIT_INSTALL_DIR}/${DIR_BASENAME}
chmod 755 ${INIT_INSTALL_DIR}/${DIR_BASENAME}

if [ -n "${RUNLVLDIR}" ] ; then

   # Now that we should have a working init script let's link in the runlevel scripts   
   ln -s ${INIT_INSTALL_DIR}/${DIR_BASENAME} ${RUNLVLDIR}/S99${DIR_BASENAME}
fi

# which icon are we using?
if [[ "$VERSION_5_UI" == "true" ]]; then
	DESKTOP_ICON_PATH=${TARGETDIR}/skin/icon_app.png
else
	DESKTOP_ICON_PATH=${TARGETDIR}/skin/icon_app_128x128.png
	# use custom if it exists
	if [ -f ${TARGETDIR}/skin/custom/icon_app_64x64.png ] ; then
		DESKTOP_ICON_PATH=${TARGETDIR}/skin/custom/icon_app_64x64.png
	fi
	if [ -f ${TARGETDIR}/skin/custom/icon_app_128x128.png ] ; then
		DESKTOP_ICON_PATH=${TARGETDIR}/skin/custom/icon_app_128x128.png
	fi
fi

# actual launcher will go in applications directory, but we'll create a shortcut on the desktop
LAUCHER_FILE_NAME=$(echo "${APP_BASENAME}.desktop" | awk '{print tolower($0)}')
DESKTOP_LAUNCHER="/home/${SRC_USER}/Desktop/${LAUCHER_FILE_NAME}"
# use 'su' only if we're operating as root
if [ "${USERNAME}" == "root" ] ; then
	APP_LAUNCHER="/usr/share/applications/${LAUCHER_FILE_NAME}"
	cp scripts/${APP_BASENAME}.desktop ${APP_LAUNCHER}
	sed -imod "s|Exec=|Exec=${TARGETDIR}/|" ${APP_LAUNCHER} && rm -rf ${APP_LAUNCHER}mod
	sed -imod "s|Icon=.*|Icon=${DESKTOP_ICON_PATH}|" ${APP_LAUNCHER} && rm -rf ${APP_LAUNCHER}mod
	if [ -d "/home/${SRC_USER}/Desktop" ] ; then
		su ${SRC_USER} -c "cp scripts/${APP_BASENAME}.desktop ${DESKTOP_LAUNCHER}"
		su ${SRC_USER} -c "chmod +x ${DESKTOP_LAUNCHER}"
		su ${SRC_USER} -c "sed -imod \"s|Exec=.*|Exec=${GUISCRIPT}|\" ${DESKTOP_LAUNCHER} && rm -rf ${DESKTOP_LAUNCHER}mod"
		su ${SRC_USER} -c "sed -imod \"s|Icon=.*|Icon=${DESKTOP_ICON_PATH}|\" ${DESKTOP_LAUNCHER} && rm -rf ${DESKTOP_LAUNCHER}mod"
	fi
else
	APP_LAUNCHER="/home/${SRC_USER}/.local/share/applications/${LAUCHER_FILE_NAME}"
	cp scripts/${APP_BASENAME}.desktop ${APP_LAUNCHER}
	sed -imod "s|Exec=|Exec=${TARGETDIR}/|" ${APP_LAUNCHER} && rm -rf ${APP_LAUNCHER}mod
	sed -imod "s|Icon=.*|Icon=${DESKTOP_ICON_PATH}|" ${APP_LAUNCHER} && rm -rf ${APP_LAUNCHER}mod
	if [ -d "/home/${SRC_USER}/Desktop" ] ; then
		cp scripts/${APP_BASENAME}.desktop ${DESKTOP_LAUNCHER}
		chmod +x ${DESKTOP_LAUNCHER}
		sed -imod "s|Exec=.*|Exec=${GUISCRIPT}|" ${DESKTOP_LAUNCHER} && rm -rf ${DESKTOP_LAUNCHER}mod
		sed -imod "s|Icon=.*|Icon=${DESKTOP_ICON_PATH}|" ${DESKTOP_LAUNCHER} && rm -rf ${DESKTOP_LAUNCHER}mod
	fi
fi

# Check for max_user_watches and suggest updating if necessary.  Many distros use 8192 by default
# so we use this value as a baseline.
INOTIFY_WATCHES=`cat /proc/sys/fs/inotify/max_user_watches`
if [[ $INOTIFY_WATCHES -le 8192 ]]; then
  log "Your Linux system is currently configured to watch $INOTIFY_WATCHES files in real time."
  log "We recommend using a larger value; see the CrashPlan support site for details"
fi

# Start the servce
${INITSCRIPT} start

# call out the "service has been started" by creating a pause
log "${APP_BASENAME} has been installed and the Service has been started automatically."

log "Important directories:"
log "  Installation:"
log "    ${TARGETDIR}"
log "  Logs:"
log "    ${TARGETDIR}/log"
log "  Default archive location:"
log "    ${MANIFESTDIR}"
log "  Readme:"
log "    ${TARGETDIR}/doc"

# if we installed as root make sure they see 'sudo' in front of the Engine start
SUDO_PREFIX="sudo "
if [ "${USERNAME}" != "root" ] ; then
	SUDO_PREFIX=""
fi
log "Start Scripts:"
log "  ${SUDO_PREFIX}${INITSCRIPT} start|stop"
log "  ${GUISCRIPT}"
log "Installation is complete. Thank you for installing Code42 ${APP_BASENAME}."
